# @(#) $Id: malaga.py 1.2 Wed, 06 Aug 2003 15:24:03 +0200 mxp $
""" Object-oriented Python interface to Malaga grammars

This module provides access to Malaga grammars from Python programs.
Malaga is a software package for the development and application of
grammars.  Malaga grammars are typically used for the linguistic
analysis of words and sentences of natural languages.

Basic usage: First create a MalagaGrammar object; the Malaga project
file is specified as argument:

  g = malaga.MalagaGrammar('/usr/local/lib/malaga/dmm/dmm.pro')

You can then use the grammar to analyze text (typically wordforms or
sentences):

  a = g.analyze("Hausmeister")

The result is a list containing one entry per analysis, i.e. a[0] is
the first analysis, etc.  The structure of an analysis depends on the
grammar.

Malaga data types are mapped to Python data types as follows:

  Malaga type           | Python type
  ----------------------+------------------------
  Symbol                | string
  String                | string
  Floating point number | number
  List                  | list
  Record                | dictionary

For example, the following Malaga analysis

  1: [clause_type: declarative,
      predicate: "ist",
      arguments: <[surf: "Gero",
                   function: subject],
                  [surf: "in der Stadt",
                   function: local]>]

results in the following Python structure (as printed by the pprint
module):

  [ { 'arguments': [ {'surf': '"Gero"', 'function': 'subject'},
                     {'surf': '"in der Stadt"', 'function': 'local'}],
      'clause_type': 'declarative',
      'predicate': '"ist"'}]

MalagaGrammar objects also provide the methods get_info() and
set_option() documented below.

"""

import _pymalaga

MORPHOLOGY = _pymalaga.MORPHOLOGY
SYNTAX     = _pymalaga.SYNTAX

class MalagaGrammar:
    """ This class represents a Malaga grammar.  You can load more
    than one grammar."""

    __active = ""

    def __init__(self, project):
        self.project  = project
        self.options  = {}

    def _isactive(self):
        """Internal method.  Check whether the grammar is active."""
        return self.__class__.__active == self.project

    def _active(self):
        """Internal method.  Activate the grammar."""
        if not self._isactive():
            _pymalaga.terminate()
            _pymalaga.init(self.project)
            for opt in self.options.iterkeys():
                _pymalaga.set_option(opt)
            self.__class__.__active = self.project

    def get_info(self):
        """
        get_info() returns a string with the informative message
        specified in the info lines of the Malaga project file.
        """
        self._active()
        return _pymalaga.get_info()
        
    def set_option(self, *options):
        """
        Set a Malaga option for this grammar.  One or more option
        strings can be specified, e.g.,

          g.set_option('robust-rule no', 'switch "ListOutput" no')

        See the chapter "Options" in the Malaga documentation for more
        information on options.
        """
        
        for option in options:
            self.options[option] = 0
            if self._isactive():
                _pymalaga.set_option(option)

    def analyze(self, string, analysis_type=MORPHOLOGY, quote_str=0):
        """
        Analyze a string.  The analysis_type and quote_str parameters are
        optional.  Analysis_type can be set to either malaga.MORPHOLOGY or
        malaga.SYNTAX; if unspecified, morphological analysis is assumed.
        If quote_str is 1, strings are enclosed in quotes to distinguish
        them from symbols.
        """

        self._active()
        
        return _pymalaga.analyze(string, analysis_type, quote_str)

    def __del__(self):
        if self._isactive():
            _pymalaga.terminate()
            self.__class__.__active = ""
